package com.persagy.apm.alarmservice.group.alarmrecord.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.persagy.apm.alarmservice.group.alarmrecord.enums.EnumAlarmListType;
import com.persagy.apm.alarmservice.group.alarmrecord.enums.EnumAlarmTreatMode;
import com.persagy.apm.alarmservice.group.alarmrecord.enums.EnumAlarmTreatState;
import com.persagy.apm.alarmservice.group.alarmrecord.model.ConvertAlarmRecordTool;
import com.persagy.apm.alarmservice.group.alarmrecord.model.dto.*;
import com.persagy.apm.alarmservice.group.alarmrecord.model.vo.AlarmRecordIdsItem;
import com.persagy.apm.alarmservice.group.alarmrecord.model.vo.AlarmRecordItemVO;
import com.persagy.apm.alarmservice.group.alarmrecord.model.vo.AlarmRecordListItem;
import com.persagy.apm.alarmservice.group.alarmrecord.service.AlarmRecordWrapper;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.model.group.vo.GroupInfoVO;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.BatchUpdateAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.QueryAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmrecord.UpdateAlarmRecordDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.common.QueryDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmRecordItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmItemServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmRecordServiceImpl;
import com.persagy.apm.energyalarmstarter.alarmdata.utils.DateUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.xssf.usermodel.XSSFSheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 报警记录控制层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
@Api(tags = "报警记录")
@Validated
@RestController
@RequestMapping("alarm_records")
public class AlarmRecordController {
    // alarm-data-starter中封装的服务
    @Autowired
    AlarmRecordServiceImpl alarmRecordService;
    // alarm-data-starter中封装的服务
    @Autowired
    AlarmItemServiceImpl alarmItemService;
    @Autowired
    GroupManagementService groupManagementService;
    @Autowired
    AlarmRecordWrapper alarmRecordWrapper;

    @ApiOperation(value = "当前报警列表")
    @PostMapping("/current_list")
    public CommonResult<PageList<AlarmRecordListItem>> queryCurrentAlarmRecord(
            @Valid @RequestBody PageQueryAlarmRecordDTO pageQueryAlarmRecordDTO) throws Exception {
        if (pageQueryAlarmRecordDTO.getAlarmTreatState() == null) {
            // 当前报警的默认查询条件为：报警处理状态为未处理和处理中
            pageQueryAlarmRecordDTO.setAlarmTreatState(Lists.newArrayList(
                    EnumAlarmTreatState.UN_HANDLE.getType(),
                    EnumAlarmTreatState.HANDLING.getType()
            ));
        }

        return queryAlarmRecord(pageQueryAlarmRecordDTO);
    }

    /**
     * 查询报警记录
     *
     * @param pageQueryAlarmRecordDTO 查询条件
     * @return 报警记录列表（分页）
     * @author lixing
     * @version V1.0 2021/9/20 6:32 下午
     */
    private CommonResult<PageList<AlarmRecordListItem>> queryAlarmRecord(
            PageQueryAlarmRecordDTO pageQueryAlarmRecordDTO) throws Exception {
        // 根据查询条件获取关联的设备
        List<String> relatedObjIds = getRelatedObjIds(pageQueryAlarmRecordDTO);
        // 从报警数据中台查询报警记录
        QueryDTO<QueryAlarmRecordDTO> queryDTO = new QueryDTO<>();
        queryDTO.setPage(pageQueryAlarmRecordDTO.getPage());
        queryDTO.setSize(pageQueryAlarmRecordDTO.getSize());
        QueryAlarmRecordDTO queryAlarmRecordDTO = ConvertAlarmRecordTool.INSTANCE.
                convert2QueryAlarmRecordDTO(pageQueryAlarmRecordDTO);
        queryDTO.setCriteria(queryAlarmRecordDTO);
        queryAlarmRecordDTO.setObjId(relatedObjIds);
        // 如果没有传排序字段，默认根据报警开始时间倒序排列
        List<Sort> orders = pageQueryAlarmRecordDTO.getOrders();
        if (CollectionUtils.isEmpty(orders)) {
            Sort sort = new Sort();
            sort.setColumn("triggerTime");
            sort.setAsc(false);
            orders = Lists.newArrayList(sort);
            pageQueryAlarmRecordDTO.setOrders(orders);
        }
        DmpResult<List<AlarmRecordItem>> query = alarmRecordService.query(queryDTO);
        // 报警数据中台查询结果转换为报警业务中台报警记录
        List<AlarmRecordListItem> resultList = alarmRecordWrapper.getAlarmRecordListItems(query);
        // 封装分页结果
        IPage<AlarmRecordListItem> result = new Page<>();
        result.setTotal(query.getCount());
        result.setRecords(resultList);
        return ResultHelper.multi(result);
    }


    /**
     * 获取关联的报警对象
     *
     * @param pageQueryAlarmRecordDTO 报警查询条件
     * @return 关联的报警对象id列表
     * @author lixing
     * @version V1.0 2021/9/18 7:12 下午
     */
    private List<String> getRelatedObjIds(PageQueryAlarmRecordDTO pageQueryAlarmRecordDTO) {
        // 如果查询条件中存在分组，查询分组下的设备
        List<String> groupIdList = pageQueryAlarmRecordDTO.getGroupId();
        String objName = pageQueryAlarmRecordDTO.getObjName();
        objName = objName == null ? null : objName.trim();
        /* 如果查询条件中包含项目id列表，使用查询条件中的项目查询
         * 如果查询条件中没有分组信息，获取所有分组下的设备
         */
        List<String> projectIdList = pageQueryAlarmRecordDTO.getProjectIds();
        if (CollectionUtils.isEmpty(projectIdList) && PoemsContext.getContext().getProjectId() != null) {
            projectIdList = Lists.newArrayList(PoemsContext.getContext().getProjectId());
        }

        if (CollectionUtils.isEmpty(groupIdList)) {
            List<GroupInfoVO> groupTreeDetailList = groupManagementService.queryAllLeafGroups();
            groupIdList = groupTreeDetailList.stream().map(GroupInfoVO::getId).collect(Collectors.toList());
        }

        List<EquipItem> equipList = new ArrayList<>();

        for (String groupId : groupIdList) {
            GroupInfoVO group = groupManagementService.getGroup(groupId);
            // 查询分组下的设备
            QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
            queryEquipDTO.setLocalName(objName);
            queryEquipDTO.setClassCode(group.getClassCode());
            queryEquipDTO.setCondition(group.getCondition());
            // 如果项目id列表为空，查询分组下的所有设备
            if (CollectionUtils.isEmpty(projectIdList)) {
                equipList.addAll(groupManagementService.getEquipList(queryEquipDTO));
            } else {
                for (String projectId : projectIdList) {
                    queryEquipDTO.setProjectId(projectId);
                    equipList.addAll(groupManagementService.getEquipList(queryEquipDTO));
                }
            }
        }

        return equipList.stream().map(EquipItem::getId).collect(Collectors.toList());
    }

    @ApiOperation(value = "历史报警列表")
    @PostMapping("/history_list")
    public CommonResult<PageList<AlarmRecordListItem>> queryHistoryAlarmRecord(
            @Valid @RequestBody PageQueryAlarmRecordDTO pageQueryAlarmRecordDTO) throws Exception {
        // 历史报警的默认查询条件为：报警处理状态为处理完成
        pageQueryAlarmRecordDTO.setAlarmTreatState(Lists.newArrayList(
                EnumAlarmTreatState.DONE.getType()
        ));
        return queryAlarmRecord(pageQueryAlarmRecordDTO);
    }

    @ApiOperation(value = "已忽略报警列表")
    @PostMapping("/ignored_list")
    public CommonResult<PageList<AlarmRecordListItem>> queryIgnoredAlarmRecord(
            @Valid @RequestBody PageQueryAlarmRecordDTO pageQueryAlarmRecordDTO) throws Exception {
        // 历史报警的默认查询条件为：报警处理状态为已忽略
        pageQueryAlarmRecordDTO.setAlarmTreatState(Lists.newArrayList(
                EnumAlarmTreatState.IGNORE.getType()
        ));
        return queryAlarmRecord(pageQueryAlarmRecordDTO);
    }

    @ApiOperation(value = "关闭报警")
    @PostMapping("/close")
    public CommonResult<AlarmRecordIdsItem> closeAlarmRecord(
            @Valid @RequestBody CloseAlarmRecordDTO closeAlarmRecordDTO) throws Exception {
        // 报警处理方式更新为关闭报警，报警状态更新为已完成
        String treatMode = EnumAlarmTreatMode.CLOSE.getType();
        String treatState = EnumAlarmTreatState.DONE.getType();
        List<String> recordIds = closeAlarmRecordDTO.getIds();

        return updateAlarmRecordTreatState(treatMode, treatState, recordIds);
    }

    /**
     * 更新报警记录处理状态
     *
     * @param treatMode  处理方式
     * @param treatState 处理状态
     * @param recordIds  报警记录id列表
     * @return 报警记录id列表
     * @author lixing
     * @version V1.0 2021/9/20 7:25 下午
     */
    private CommonResult<AlarmRecordIdsItem> updateAlarmRecordTreatState(
            String treatMode, String treatState, List<String> recordIds) throws Exception {
        AlarmRecordIdsItem alarmRecordIdsItem = new AlarmRecordIdsItem();

        if (CollectionUtils.isEmpty(recordIds)) {
            return ResultHelper.single(alarmRecordIdsItem);
        }

        List<UpdateAlarmRecordDTO> updateAlarmRecordDTOList = new ArrayList<>();
        for (String recordId : recordIds) {
            UpdateAlarmRecordDTO updateAlarmRecordDTO = new UpdateAlarmRecordDTO();
            updateAlarmRecordDTO.setId(recordId);
            updateAlarmRecordDTO.setTreatMode(Integer.valueOf(treatMode));
            updateAlarmRecordDTO.setTreatState(Integer.valueOf(treatState));
            updateAlarmRecordDTOList.add(updateAlarmRecordDTO);
            alarmRecordIdsItem.setIds(recordIds);
        }
        BatchUpdateAlarmRecordDTO batchUpdateAlarmRecordDTO = new BatchUpdateAlarmRecordDTO();
        batchUpdateAlarmRecordDTO.setAlarmRecordModels(updateAlarmRecordDTOList);

        alarmRecordService.batchUpdate(batchUpdateAlarmRecordDTO);
        return ResultHelper.single(alarmRecordIdsItem);
    }

    @ApiOperation(value = "转工单")
    @PostMapping("/convert_work_order")
    public CommonResult<AlarmRecordIdsItem> alarmRecordConvertWorkOrder(
            @Valid @RequestBody AlarmRecordConvertWorkOrderDTO alarmRecordConvertWorkOrderDTO) throws Exception {
        // 报警处理方式更新为转工单，报警状态更新为处理中
        String treatMode = EnumAlarmTreatMode.ORDER.getType();
        String treatState = EnumAlarmTreatState.HANDLING.getType();
        List<String> recordIds = alarmRecordConvertWorkOrderDTO.getIds();

        return updateAlarmRecordTreatState(treatMode, treatState, recordIds);
    }

    @ApiOperation(value = "忽略")
    @PostMapping("/ignore")
    public CommonResult<AlarmRecordIdsItem> ignoreAlarmRecord(
            @Valid @RequestBody IgnoreAlarmRecordDTO ignoreAlarmRecordDTO) throws Exception {
        // 报警处理方式更新为忽略，报警状态更新为已忽略
        String treatMode = EnumAlarmTreatMode.IGNORE.getType();
        String treatState = EnumAlarmTreatState.IGNORE.getType();
        List<String> recordIds = ignoreAlarmRecordDTO.getIds();

        return updateAlarmRecordTreatState(treatMode, treatState, recordIds);
    }

    @ApiOperation(value = "报警详情")
    @PostMapping("/get")
    public CommonResult<AlarmRecordItemVO> getAlarmRecord(
            @Valid @RequestBody GetAlarmRecordDTO getAlarmRecordDTO) throws Exception {
        AlarmRecordItem alarmRecord = alarmRecordWrapper.getAlarmRecord(getAlarmRecordDTO.getId());
        AlarmRecordItemVO alarmRecordItemVO = alarmRecordWrapper.convert2AlarmRecordItemVO(alarmRecord);
        // 报警触发条件
        return ResultHelper.single(alarmRecordItemVO);
    }

    @ApiOperation(value = "下载")
    @PostMapping("/download")
    public void downloadAlarmRecord(
            @Valid @RequestBody DownloadAlarmRecordDTO downloadAlarmRecordDTO,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        //获取报警id列表
        List<String> recordIdList = downloadAlarmRecordDTO.getIds();
        //列表类型
        String listType = downloadAlarmRecordDTO.getListType();
        EnumAlarmListType enumAlarmListType = EnumAlarmListType.getByType(listType);
        if (enumAlarmListType == null) {
            throw new IllegalArgumentException("列表类型传值错误");
        }
        // 获取报警详情列表
        List<AlarmRecordItem> alarmRecordItemList = alarmRecordWrapper.getAlarmRecordList(recordIdList);

        // 创建Workbook
        XSSFWorkbook wb = new XSSFWorkbook();
        XSSFSheet overViewSheet = wb.createSheet();
        // 设置sheet页标题
        wb.setSheetName(0, enumAlarmListType.getDesc());
        // 处理表头
        alarmRecordWrapper.handleExcelTitle(enumAlarmListType, overViewSheet, wb);

        if (!CollectionUtils.isEmpty(alarmRecordItemList) && alarmRecordItemList.size() > 0) {
            List<AlarmRecordItemVO> alarmRecordItemVOList = new ArrayList<>();
            for (AlarmRecordItem alarmRecord : alarmRecordItemList) {
                alarmRecordItemVOList.add(alarmRecordWrapper.convert2AlarmRecordItemVO(alarmRecord));
            }
            // 能耗概览Sheet数据填充
            alarmRecordWrapper.fillDataTable(enumAlarmListType, overViewSheet, alarmRecordItemVOList);
        }
        String time = DateUtils.date2Str(new Date(), DateUtils.SDFSECOND);
        String excelName = enumAlarmListType.getDesc() + "-" + time + ".xlsx";

        response.reset();
        response.setContentType("application/msexcel;charset=UTF-8");
        response.setCharacterEncoding("UTF-8");
        try {
            excelName = URLEncoder.encode(excelName, "UTF-8");
            response.setHeader("Content-disposition", "attachment;filename=" + excelName + ";" +
                    "filename*=utf-8''" + excelName);
            OutputStream out = response.getOutputStream();
            wb.write(out);
            out.flush();
            out.close();
            wb.close();
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}

