package com.persagy.apm.alarmservice.group.management.service;

import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipListItem;
import com.persagy.apm.alarmservice.group.management.model.group.vo.EquipGroupInfoVO;
import com.persagy.apm.alarmservice.group.management.model.group.vo.GroupInfoVO;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.common.response.PageList;

import java.util.List;
import java.util.Map;

/**
 * @author 易涛
 * @version 1.0
 * @date 2021/9/8 14:17
 */
public interface GroupManagementService {
    /**
     * 分组条件（设备类+查询条件）+项目 获取设备
     *
     * @param queryDTO 查询条件
     * @return 设备列表（带分页）
     * @author 易涛
     * @date 2021/9/8 16:54
     */
    PageList<EquipItem> equipsQuery(QueryEquipDTO queryDTO);

    /**
     * 获取设备详情
     *
     * @param id 设备id
     * @return 设备详细信息
     * @author 易涛
     * @date 2021/9/9 16:04
     */
    EquipItem equipGet(String id);

    /**
     * 根据一组id查询设备
     *
     * @param equipIds 设备id列表
     * @return 设备id -> 设备信息 map
     * @author lixing
     * @version V1.0 2021/9/20 1:15 下午
     */
    Map<String, EquipItem> queryEquipsByIds(List<String> equipIds);

    /**
     * 根据一组设备id查询设备和所在分组信息
     *
     * @param equipIds 设备id数组
     * @return 设备和所在分组信息列表
     * @author 易涛
     * @date 2021/9/10 10:02
     */
    List<EquipGroupInfoVO> equipGroupQuery(List<String> equipIds);

    /**
     * 根据一组设备id查询对应的分组信息
     *
     * @param objIds 设备id列表
     * @return 设备id->分组信息映射
     * @author lixing
     * @version V1.0 2021/9/20 1:14 下午
     */
    Map<String, SimpleObjVO> queryObjGroup(List<String> objIds);

    /**
     * 定时任务删除valid为0的数据
     *
     * @return void
     * @author 易涛
     * @date 2021/9/14 15:55
     */
    void batchDelete();

    /**
     * 查询分组下的设备
     *
     * @param condition 分组查询条件
     * @param classCode 设备类
     * @return 分组下的设备
     */
    List<EquipItem> getEquipList(String condition, String classCode);

    /**
     * 使用默认分页条件查询分组下的设备
     *
     * @param queryEquipDTO 查询条件
     * @return 分组下的设备
     */
    List<EquipItem> getEquipList(QueryEquipDTO queryEquipDTO);

    /**
     * 获取分组信息
     *
     * @param id 分组id
     * @return 分组信息
     * @author lixing
     * @version V1.0 2021/9/18 7:03 下午
     */
    GroupInfoVO getGroup(String id);

    /**
     * 查询所有叶子节点分组
     *
     * @return 所有可用分组的列表
     * @author lixing
     * @version V1.0 2021/9/19 1:02 下午
     */
    List<GroupInfoVO> queryAllLeafGroups();
}
