package com.persagy.apm.alarmservice.project.alarmcondition.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.alarmservice.group.calculatemethod.model.dto.CalculateMethodDTO;
import com.persagy.apm.alarmservice.group.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.group.monitorindicator.service.IMonitorIndicatorService;
import com.persagy.apm.alarmservice.project.alarmcondition.dao.AlarmConditionMapper;
import com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition;
import com.persagy.apm.alarmservice.project.alarmcondition.model.ConvertAlarmConditionTool;
import com.persagy.apm.alarmservice.project.alarmcondition.model.Formula;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.common.model.dto.Sort;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.common.enums.ResponseCode;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;
import java.util.regex.Pattern;

/**
 * 报警条件(AlarmCondition) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:30:38
 */
@Service
public class AlarmConditionServiceImpl extends ServiceImpl<AlarmConditionMapper, AlarmCondition>
        implements IAlarmConditionService {

    @Autowired
    private IMonitorIndicatorService monitorIndicatorService;

    /**
     * 创建报警条件
     *
     * @return 报警条件主键
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public Map<String, Object> createAlarmCondition(AddAlarmConditionDTO addAlarmConditionDTO) {
        AlarmCondition alarmCondition = ConvertAlarmConditionTool.INSTANCE.convertAddDto2Entity(addAlarmConditionDTO);
        List<List<CalculateMethodDTO>> frontend = addAlarmConditionDTO.getFrontend();
        alarmCondition.setFrontend(frontend);
        //将前端公式转换成后台公式
        Map<String, Object> stringMap = frontend2Backend(frontend);
        alarmCondition.setBackend((String) stringMap.get("backend"));
        alarmCondition.setInfoCodes((List<String>) stringMap.get("infoCode"));
        // 设置默认值
        setDefaultValue(alarmCondition);
        save(alarmCondition);
        stringMap.put("id", alarmCondition.getId());
        return stringMap;
    }

    @Override
    public void saveCondition(AlarmCondition alarmCondition){
        save(alarmCondition);
    }

    /**
     * 转换报警条件
     *
     * @return 报警条件主键
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public Map<String, Object> swtichAlarmCondition(AddAlarmConditionDTO addAlarmConditionDTO) {
        AlarmCondition alarmCondition = ConvertAlarmConditionTool.INSTANCE.convertAddDto2Entity(addAlarmConditionDTO);
        List<List<CalculateMethodDTO>> frontend = addAlarmConditionDTO.getFrontend();
        alarmCondition.setFrontend(frontend);
        //将前端公式转换成后台公式
        Map<String, Object> stringMap = frontend2Backend(frontend);
        alarmCondition.setBackend((String) stringMap.get("backend"));
        alarmCondition.setInfoCodes((List<String>) stringMap.get("infoCode"));
        // 设置默认值
        setDefaultValue(alarmCondition);
        stringMap.put("alarmCondition",alarmCondition);
        stringMap.put("id", alarmCondition.getId());
        return stringMap;
    }

    public Map<String, Object> frontend2Backend(List<List<CalculateMethodDTO>> frontend) {
        StringBuffer allNotNull = new StringBuffer();
        StringBuffer allNull = new StringBuffer();
        Set<String> infoCodesSet = new HashSet<>();
        Set<String> displayFormulaList = new HashSet<>();
        for (List<CalculateMethodDTO> calculateMethodDTOS : frontend) {
            StringBuffer notNullString = new StringBuffer();
            StringBuffer nullString = new StringBuffer();
            StringBuffer calculateString = new StringBuffer();
            for (CalculateMethodDTO calculateMethodDTO : calculateMethodDTOS) {
                infoCodesSet.add(calculateMethodDTO.getMonitorIndicatorId());
                List<Formula> formulas = calculateMethodDTO.getFormula();
                StringBuffer tempFormula = new StringBuffer();
                StringBuffer displayFormula = new StringBuffer();
                displayFormula.append(calculateMethodDTO.getMonitorIndicatorName()).append(calculateMethodDTO.getOperator());
                for (Formula formula : formulas) {
                    if (StringUtils.isNotBlank(formula.getValue())) {
                        nullString.append(formula.getValue()).append("==NaN").append(" || ");
                        notNullString.append(formula.getValue()).append("!=NaN").append(" && ");
                        tempFormula.append(formula.getValue());
                        infoCodesSet.add(formula.getValue());
                    } else {
                        tempFormula.append(replace(formula.getLabel()));
                    }
                    displayFormula.append(formula.getLabel());
                }
                displayFormulaList.add(displayFormula.toString());
                calculateMethodDTO.setTempFormula(tempFormula.toString());
                nullString.append(calculateMethodDTO.getMonitorIndicatorId()).append("==NaN").append(" || ");
                notNullString.append(calculateMethodDTO.getMonitorIndicatorId()).append("!=NaN").append(" && ");
                calculateString.append("(")
                        .append(calculateMethodDTO.getMonitorIndicatorId())
                        .append(replace(calculateMethodDTO.getOperator()))
                        .append(calculateMethodDTO.getTempFormula()).append(")")
                        .append(" && ");
            }
            notNullString.delete(notNullString.length() - 4, notNullString.length()).append(")");
            allNotNull.append("(").append(notNullString).append(" || ");
            allNull.append("(").append("(").append(nullString.substring(0, nullString.length() - 4)).append(")").append("?true:").append("(").append(calculateString.substring(0, calculateString.length() - 4)).append(")").append(")").append(" || ");

        }
        StringBuffer substring = new StringBuffer();
        substring.append("(").append(allNotNull.substring(0, allNotNull.length() - 4)).append(")").append(" && ").append("(").append(allNull.substring(0, allNull.length() - 4)).append(")").toString();
        Map<String, Object> map = new HashMap();
        List<String> infoCodesList = new ArrayList<>();
        infoCodesList.addAll(infoCodesSet);
        List<String> displayFormula = new ArrayList();
        displayFormula.addAll(displayFormulaList);
        map.put("infoCode", infoCodesList);
        map.put("backend", substring.toString());
        map.put("end", "!" + substring);
        map.put("displayFormula", displayFormula);
        return map;
    }

    public String replace(String operator) {
        operator = operator.replace("=", "==")
                .replace("≥", ">=")
                .replace("≤", "<=");
        return operator;
    }

    @Override
    public void updateById(Map<String, Object> conditionMap, UpdateAlarmConditionDTO condition) {
        AlarmCondition alarmCondition = new AlarmCondition();
        alarmCondition.setFrontend(condition.getFrontend());
        alarmCondition.setAlarmLevel(condition.getAlarmLevel());
        alarmCondition.setBackend((String) conditionMap.get("backend"));
        alarmCondition.setInfoCodes((List<String>) conditionMap.get("infoCode"));
        alarmCondition.setTriggerUphold(condition.getTriggerUphold());
        alarmCondition.setId(condition.getId());
        updateById(alarmCondition);
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param alarmCondition 报警条件实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(AlarmCondition alarmCondition) {
        alarmCondition.setCreator(PoemsContext.getContext().getUserId());
    }

    /**
     * 报警条件详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public AlarmCondition queryAlarmConditionDetail(String id) {
        AlarmCondition alarmCondition = getById(id);
        if (alarmCondition == null) {
            throw new IllegalArgumentException("查看AlarmCondition详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return alarmCondition;
    }

    /**
     * 更新报警条件
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public void updateAlarmCondition(UpdateAlarmConditionDTO updateAlarmConditionDTO) {
        AlarmCondition alarmCondition = getById(updateAlarmConditionDTO.getId());
        alarmCondition = ConvertAlarmConditionTool.INSTANCE.convertUpdateDto2Entity(alarmCondition, updateAlarmConditionDTO);
        alarmCondition.setModifier(PoemsContext.getContext().getUserId());
        if (updateAlarmConditionDTO.getFrontend() != null) {
            List<List<CalculateMethodDTO>> frontend = updateAlarmConditionDTO.getFrontend();
            alarmCondition.setFrontend(frontend);
            //将前端公式转换成后台公式
            Map<String, Object> stringMap = frontend2Backend(frontend);
            alarmCondition.setBackend((String) stringMap.get("backend"));
            alarmCondition.setInfoCodes((List<String>) stringMap.get("infoCode"));
        }
        updateById(alarmCondition);
    }

    /**
     * 校验报警条件是否可删除
     *
     * @param id 报警条件主键
     * @return 报警条件do类
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    public AlarmCondition checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除AlarmCondition时发生异常，主键为空");
        }

        AlarmCondition alarmCondition = getById(id);

        if (alarmCondition == null) {
            throw new IllegalArgumentException("删除AlarmCondition时发生异常，找不到要删除的数据，id:" + id);
        }

        return alarmCondition;
    }

    /**
     * 删除报警条件
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public void deleteAlarmCondition(String id) {
        // 校验是否可删除
        AlarmCondition alarmCondition = checkDeletable(id);
        alarmCondition.setValid(ValidEnum.FALSE.getType());
        updateById(alarmCondition);
    }

    /**
     * 查询报警条件
     *
     * @return List<AlarmCondition>
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public List<AlarmCondition> queryAlarmConditionList(QueryAlarmConditionDTO queryAlarmConditionDTO) {
        QueryWrapper<AlarmCondition> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(AlarmCondition.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, AlarmCondition.PROP_CREATIONTIME);

        if (queryAlarmConditionDTO != null) {

            if (queryAlarmConditionDTO.getInfoCodes() != null) {
                queryWrapper.eq(AlarmCondition.PROP_INFO_CODES, queryAlarmConditionDTO.getInfoCodes());
            }

            if (queryAlarmConditionDTO.getFrontend() != null) {
                queryWrapper.eq(AlarmCondition.PROP_FRONTEND, queryAlarmConditionDTO.getFrontend());
            }

            if (queryAlarmConditionDTO.getBackend() != null) {
                queryWrapper.eq(AlarmCondition.PROP_BACKEND, queryAlarmConditionDTO.getBackend());
            }

            if (queryAlarmConditionDTO.getTriggerUphold() != null) {
                queryWrapper.eq(AlarmCondition.PROP_TRIGGER_UPHOLD, queryAlarmConditionDTO.getTriggerUphold());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryAlarmConditionDTO.getAlarmLevel())) {
                queryWrapper.like(AlarmCondition.PROP_ALARM_LEVEL, queryAlarmConditionDTO.getAlarmLevel());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询报警条件
     *
     * @return IPage<AlarmCondition>
     * @author lixing
     * @version V1.0 2021-09-08 22:30:38
     */
    @Override
    public IPage<AlarmCondition> pageQueryAlarmCondition(PageQueryAlarmConditionDTO pageQueryAlarmConditionDTO) {
        QueryWrapper<AlarmCondition> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(AlarmCondition.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<AlarmCondition> pageParam = new Page<>(pageQueryAlarmConditionDTO.getPage(), pageQueryAlarmConditionDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryAlarmConditionDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, AlarmCondition.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryAlarmConditionDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        if (pageQueryAlarmConditionDTO.getInfoCodes() != null) {
            queryWrapper.eq(AlarmCondition.PROP_INFO_CODES, pageQueryAlarmConditionDTO.getInfoCodes());
        }

        if (pageQueryAlarmConditionDTO.getFrontend() != null) {
            queryWrapper.eq(AlarmCondition.PROP_FRONTEND, pageQueryAlarmConditionDTO.getFrontend());
        }

        if (pageQueryAlarmConditionDTO.getBackend() != null) {
            queryWrapper.eq(AlarmCondition.PROP_BACKEND, pageQueryAlarmConditionDTO.getBackend());
        }

        if (pageQueryAlarmConditionDTO.getTriggerUphold() != null) {
            queryWrapper.eq(AlarmCondition.PROP_TRIGGER_UPHOLD, pageQueryAlarmConditionDTO.getTriggerUphold());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryAlarmConditionDTO.getAlarmLevel())) {
            queryWrapper.like(AlarmCondition.PROP_ALARM_LEVEL, pageQueryAlarmConditionDTO.getAlarmLevel());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public CommonResult<Boolean> validate(ValidateAlarmConditionDTO validateAlarmConditionDTO) {
        MonitorIndicator monitorIndicator = monitorIndicatorService.queryMonitorIndicatorDetail(validateAlarmConditionDTO.getFrontend().getMonitorIndicatorId());
        List<Formula> formulas = validateAlarmConditionDTO.getFrontend().getFormula();
        StringBuffer displayFormula = new StringBuffer();
        String infoCodes = "";
        for (Formula formula : formulas) {
            if (formula.getValue() != null) {
                String value = formula.getValue();
                infoCodes = value;
                displayFormula.append(value);
            } else {
                displayFormula.append(formula.getLabel());
            }
        }
        if (isNumeric(displayFormula.toString())) {
            if (formulas.size() == 1) {
                return ResultHelper.success();
            } else {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
            }
        }
        if (!"".equals(infoCodes)) {
            MonitorIndicator indicator = monitorIndicatorService.queryMonitorIndicatorDetail(infoCodes);
            String monitorUnit = monitorIndicator.getUnit();
            String unit = indicator.getUnit();
            if (monitorUnit != null && unit != null && !monitorUnit.equals(unit)) {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "选择的指标与单位指标不一致");
            } else if (monitorUnit == null || unit == null) {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式错误");
            }
            return this.validate(displayFormula.toString(), infoCodes);
        }

        return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
    }

    public static boolean isNumeric(String str) {
        for (int i = 0; i < str.length(); i++) {
            System.out.println(str.charAt(i));
            if (!Character.isDigit(str.charAt(i))) {
                return false;
            }
        }
        return true;
    }

    /**
     * 使用正则来校验数学公式
     *
     * @param expression 数学公式，包含变量
     * @param
     * @return
     */
    public static CommonResult validate(String expression, String variable) {
//        if (variables == null || variables.isEmpty()) {
//            return ResultHelper.failure(ResponseCode.C0134.getCode(), "内置变量为空");
//        }
        // 去空格
        expression = expression.replaceAll(" ", "");
        // 连续运算符处理
        if (expression.split("[\\+\\-\\*\\/]{2,}").length > 1) {
            return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
        }
        if (org.apache.commons.lang3.StringUtils.contains(expression, "()")) {
            return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
        }
//        expression = expression.replaceAll("\\)\\(", "\\)*\\(");
//        expression = expression.replaceAll("\\(\\-", "\\(0-");
//        expression = expression.replaceAll("\\(\\+", "\\(0+");
        // 校验变量
        String[] splits = expression.split("\\+|\\-|\\*|\\/|\\(|\\)");
        for (String split : splits) {
            if (org.apache.commons.lang3.StringUtils.isBlank(split) || Pattern.matches("[0-9]+", split)) {
                continue;
            }
            if (!variable.equals(split)) {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
            }
        }
        // 校验括号
        Character preChar = null;
        Stack<Character> stack = new Stack<>();
        String resultExpression = expression;
        for (int i = 0; i < expression.length(); i++) {
            char currChar = expression.charAt(i);
            if (i == 0) {
                if (Pattern.matches("\\*|\\/", String.valueOf(currChar))) {
                    return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
                }
                if (currChar == '+') {
                    resultExpression = expression.substring(1);
                }
                if (currChar == '-') {
                    resultExpression = "0" + expression;
                }
            }
            if ('(' == currChar) {
                stack.push('(');
            } else if (')' == currChar) {
                if (stack.size() > 0) {
                    stack.pop();
                } else {
                    return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
                }
            }
            if (preChar != null && preChar == '(' && Pattern.matches("[\\+\\-\\*\\/]+", String.valueOf(currChar))) {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
            }
            if (preChar != null && preChar == ')' && !Pattern.matches("[\\+\\-\\*\\/]+", String.valueOf(currChar))) {
                return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
            }
            if (i == expression.length() - 1) {
                if (Pattern.matches("\\+|\\-|\\*|\\/", String.valueOf(currChar))) {
                    return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
                }
            }
            preChar = currChar;
        }
        if (stack.size() > 0) {
            return ResultHelper.failure(ResponseCode.C0134.getCode(), "公式不合法");
        }
        return ResultHelper.success();
    }
}
