package com.persagy.apm.alarmservice.project.alarmrule.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.persagy.apm.alarmservice.common.configuration.rabbitmq.RabbitMsgSender;
import com.persagy.apm.alarmservice.common.model.DmpMessage;
import com.persagy.apm.alarmservice.common.util.DataUtils;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.AddGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.GetGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.QueryGroupAlarmRuleDTO;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.group.management.model.equip.dto.QueryEquipDTO;
import com.persagy.apm.alarmservice.group.management.model.equip.vo.EquipItem;
import com.persagy.apm.alarmservice.group.management.service.impl.GroupManagementServiceImpl;
import com.persagy.apm.alarmservice.project.alarmcondition.model.ConvertAlarmConditionTool;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.AddAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.model.dto.UpdateAlarmConditionDTO;
import com.persagy.apm.alarmservice.project.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.project.alarmrule.dao.ProjectAlarmRuleMapper;
import com.persagy.apm.alarmservice.project.alarmrule.model.ConvertProjectAlarmRuleTool;
import com.persagy.apm.alarmservice.project.alarmrule.model.ProjectAlarmRule;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.*;
import com.persagy.apm.alarmservice.project.alarmrule.model.vo.ProjectAlarmRuleListItemVO;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.dao.ProjectAlarmRuleObjRelMapper;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ProjectAlarmRuleObjRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.AddProjectAlarmRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.QueryRuleObjRelDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.dto.UpdateAlarmItemDTO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.vo.ProjectVO;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.context.poems.PoemsContext;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.AlarmCondition;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.AlarmEffectTime;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.*;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static org.apache.commons.lang3.math.NumberUtils.INTEGER_ONE;

/**
 * 项目报警规则(ProjectAlarmRule) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:46:16
 */
@Service
public class ProjectAlarmRuleServiceImpl extends ServiceImpl<ProjectAlarmRuleMapper, ProjectAlarmRule>
        implements IProjectAlarmRuleService {

    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IGroupAlarmRuleService groupAlarmRuleService;
    @Autowired
    private GroupManagementServiceImpl groupManagementService;
    @Autowired
    private AlarmConfigServiceImpl alarmConfigService;
    @Autowired
    private ProjectAlarmRuleMapper mapper;
    @Autowired
    private RabbitMsgSender rabbitMsgSender;
    @Autowired
    private ProjectAlarmRuleObjRelMapper projectAlarmRuleObjRelMapper;

    /**
     * 创建项目报警规则
     *
     * @return 项目报警规则主键
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public String createProjectAlarmRule(AddProjectAlarmRuleDTO addProjectAlarmRuleDTO) throws Exception {
        ProjectAlarmRule projectAlarmRule = ConvertProjectAlarmRuleTool.INSTANCE.convertAddDto2Entity(addProjectAlarmRuleDTO);
        AddAlarmConditionDTO alarmConditionDTO = addProjectAlarmRuleDTO.getAlarmCondition();
        ConvertAlarmConditionTool.INSTANCE.convertAddDto2Entity(alarmConditionDTO);
        //存储报警规则
        Map<String, Object> alarmConditionMap = alarmConditionService.createAlarmCondition(alarmConditionDTO);
        String conditionId = (String) alarmConditionMap.get("id");
        projectAlarmRule.setAlarmConditionId(conditionId);
        projectAlarmRule.setId(DataUtils.getUUID());
        setDefaultValue(projectAlarmRule);
        //存储项目报警
        save(projectAlarmRule);

        List<ProjectAlarmRuleObjRel> addEquipList = addProjectAlarmRuleDTO.getAddEquipList();
        addEquipList.forEach(addEquip -> {
            addEquip.setProjectAlarmRuleId(projectAlarmRule.getId());
        });
        //保存报警对象
        projectAlarmRuleObjRelService.saveProjectAlarmRule(addEquipList);
        //保存中台规则
        BatchAddAlarmConfigDTO batchAddAlarmConfigDTO = new BatchAddAlarmConfigDTO();
        List<AddAlarmConfigDTO> alarmConfigs = new ArrayList<>();
        for (ProjectAlarmRuleObjRel ProjectAlarmRuleObjRel : addEquipList) {
            AddAlarmConfigDTO addAlarmConfigDTO = new AddAlarmConfigDTO();
            String equipId = ProjectAlarmRuleObjRel.getObjId();
            addAlarmConfigDTO.setObjId(equipId);
            addAlarmConfigDTO.setItemCode(addProjectAlarmRuleDTO.getAlarmItemCode());
            addAlarmConfigDTO.setClassCode(addProjectAlarmRuleDTO.getClassCode());
            addAlarmConfigDTO.setLevel(alarmConditionDTO.getAlarmLevel());
            addAlarmConfigDTO.setCategory("Eqdx");
            addAlarmConfigDTO.setOpen(1);
            AlarmCondition equipCondition = new AlarmCondition();
            List<String> infoCodeList = (List<String>) alarmConditionMap.get("infoCode");
            List<String> infoCodeNew = new ArrayList<>();
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "_" + infoCode;
                infoCodeNew.add(infoCode.replaceAll(infoCode, replacement));
            }
            equipCondition.setInfoCode(infoCodeNew);
            String backend = (String) alarmConditionMap.get("backend");
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "_" + infoCode;
                backend = backend.replaceAll(infoCode, replacement);
            }
            equipCondition.setEnd("!" + backend);
            equipCondition.setEndUphold(0);
            equipCondition.setTrigger(backend);
            equipCondition.setTriggerUphold(alarmConditionDTO.getTriggerUphold() * 60);
            equipCondition.setRules((List<String>) alarmConditionMap.get("displayFormula"));
            JSONArray configs = JSONArray.parseArray(JSON.toJSONString(alarmConditionDTO.getFrontend()));
            equipCondition.setConfigs(configs);
            AlarmEffectTime effectTime = new AlarmEffectTime();
            effectTime.setType("always");
            equipCondition.setEffectTime(effectTime);
            addAlarmConfigDTO.setCondition(equipCondition);
            alarmConfigs.add(addAlarmConfigDTO);
        }
        sendBatchCreateAlarmConfigMsg(alarmConfigs, addProjectAlarmRuleDTO.getProjectId());
        return projectAlarmRule.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param projectAlarmRule 项目报警规则实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(ProjectAlarmRule projectAlarmRule) {
        projectAlarmRule.setCreator(AppContext.getContext().getAccountId());
    }

    /**
     * 项目报警规则详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public ProjectAlarmRule queryProjectAlarmRuleDetail(String id) {
        ProjectAlarmRule projectAlarmRule = getById(id);
        if (projectAlarmRule == null) {
            throw new IllegalArgumentException("查看ProjectAlarmRule详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return projectAlarmRule;
    }

    /**
     * 更新项目报警规则
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateProjectAlarmRule(UpdateProjectAlarmRuleDTO updateProjectAlarmRuleDTO) throws Exception {
        ProjectAlarmRule projectAlarmRule = getById(updateProjectAlarmRuleDTO.getId());
        projectAlarmRule.setModifier(AppContext.getContext().getAccountId());
        AddAlarmConditionDTO alarmCondition = updateProjectAlarmRuleDTO.getAlarmCondition();
        Map<String, Object> conditionMap = alarmConditionService.swtichAlarmCondition(alarmCondition);
        //查询已有项目报警
        int belongOld = projectAlarmRule.getBelong();
        int belongNew = updateProjectAlarmRuleDTO.getBelong();
        if (belongOld > belongNew) {//集团规则->项目自定义规则
            //新增规则
            com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition condition =
                    (com.persagy.apm.alarmservice.project.alarmcondition.model.AlarmCondition) conditionMap.get("alarmCondition");
            String uuid = DataUtils.getUUID();
            condition.setId(uuid);
            alarmConditionService.saveCondition(condition);
            //将规则ID传给项目报警
            projectAlarmRule.setAlarmConditionId(uuid);
        } else if (belongOld < belongNew) {//项目自定义规则->集团规则
            //删除自定义规则
            ProjectAlarmRule projectAlarm = getById(updateProjectAlarmRuleDTO.getId());
            alarmConditionService.deleteAlarmCondition(projectAlarm.getAlarmConditionId());
            //查询集团规则，将集团规则ID传给项目报警
            GetGroupAlarmRuleDTO getGroupAlarmRuleDTO = new GetGroupAlarmRuleDTO();
            getGroupAlarmRuleDTO.setGroupId(updateProjectAlarmRuleDTO.getGroupId());
            getGroupAlarmRuleDTO.setAlarmItemCode(updateProjectAlarmRuleDTO.getAlarmItemCode());
            GroupAlarmRule groupAlarmRule = groupAlarmRuleService.getGroupAlarmRule(getGroupAlarmRuleDTO);
            projectAlarmRule.setAlarmConditionId(groupAlarmRule.getId());
        } else if (belongOld == 0 && belongNew == 0) {//项目自定义规则->项目自定义规则
            //修改自定义规则
            UpdateAlarmConditionDTO updateAlarmConditionDTO = new UpdateAlarmConditionDTO();
            updateAlarmConditionDTO.setFrontend(alarmCondition.getFrontend());
            updateAlarmConditionDTO.setId(updateProjectAlarmRuleDTO.getAlarmConditionId());
            updateAlarmConditionDTO.setTriggerUphold(alarmCondition.getTriggerUphold());
            updateAlarmConditionDTO.setAlarmLevel(alarmCondition.getAlarmLevel());
            alarmConditionService.updateAlarmCondition(updateAlarmConditionDTO);
        }
        //更新项目报警
        projectAlarmRule.setModifiedTime(new Date());
        projectAlarmRule.setModifier(PoemsContext.getContext().getUserId());
        updateById(projectAlarmRule);
        //删除报警对象关联关系
        //projectAlarmRuleObjRelService.deleteRelation(updateProjectAlarmRuleDTO.getId());
        projectAlarmRuleObjRelMapper.deleteByProjectAlarmRuleId(updateProjectAlarmRuleDTO.getId());
        //新增关联关系
        List<UpdateAlarmItemDTO> addRelationList = updateProjectAlarmRuleDTO.getAddEquipList();
        String projectId = updateProjectAlarmRuleDTO.getProjectId();
        String id = updateProjectAlarmRuleDTO.getId();
        List<AddProjectAlarmRuleObjRelDTO> addRelations = convert2AddProjectAlarmRuleObjRelDTO(addRelationList, id, projectId);
        projectAlarmRuleObjRelService.createBatchRelation(addRelations);
        String condition = updateProjectAlarmRuleDTO.getCondition();
        String classCode = updateProjectAlarmRuleDTO.getClassCode();
        String alarmItemCode = updateProjectAlarmRuleDTO.getAlarmItemCode();
        //删除中台报警条件
        groupAlarmRuleService.deleteEquipCondition(condition, classCode, alarmItemCode);
        //新增中台报警条件
        List<AddAlarmConfigDTO> alarmConfigs = new ArrayList<>();
        for (AddProjectAlarmRuleObjRelDTO equipRel : addRelations) {
            AddAlarmConfigDTO addAlarmConfigDTO = new AddAlarmConfigDTO();
            String equipId = equipRel.getObjId();
            addAlarmConfigDTO.setObjId(equipId);
            addAlarmConfigDTO.setItemCode(alarmItemCode);
            addAlarmConfigDTO.setClassCode(classCode);
            addAlarmConfigDTO.setLevel(alarmCondition.getAlarmLevel());
            addAlarmConfigDTO.setCategory("Eqdx");
            addAlarmConfigDTO.setOpen(1);
            AlarmCondition equipCondition = new AlarmCondition();
            List<String> infoCodeList = (List<String>) conditionMap.get("infoCode");
            List<String> infoCodeNew = new ArrayList<>();
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "_" + infoCode;
                infoCodeNew.add(infoCode.replaceAll(infoCode, replacement));
            }
            equipCondition.setInfoCode(infoCodeNew);
            String backend = (String) conditionMap.get("backend");
            for (String infoCode : infoCodeList) {
                String replacement = equipId + "_" + infoCode;
                backend = backend.replaceAll(infoCode, replacement);
            }
            equipCondition.setEnd("!" + backend);
            equipCondition.setEndUphold(0);
            equipCondition.setTrigger(backend);
            equipCondition.setTriggerUphold(alarmCondition.getTriggerUphold() * 60);
            equipCondition.setRules((List<String>) conditionMap.get("displayFormula"));
            JSONArray configs = JSONArray.parseArray(JSON.toJSONString(alarmCondition.getFrontend()));
            equipCondition.setConfigs(configs);
            AlarmEffectTime effectTime = new AlarmEffectTime();
            effectTime.setType("always");
            equipCondition.setEffectTime(effectTime);
            addAlarmConfigDTO.setCondition(equipCondition);
            alarmConfigs.add(addAlarmConfigDTO);
        }
        sendBatchCreateAlarmConfigMsg(alarmConfigs, projectId);
    }

    private List<AddProjectAlarmRuleObjRelDTO> convert2AddProjectAlarmRuleObjRelDTO(List<UpdateAlarmItemDTO> addRelationList, String id, String projectId) {
        List<AddProjectAlarmRuleObjRelDTO> list = new ArrayList<>();
        for (UpdateAlarmItemDTO updateAlarmItemDTO : addRelationList) {
            AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO = new AddProjectAlarmRuleObjRelDTO();
            addProjectAlarmRuleObjRelDTO.setProjectAlarmRuleId(id);
            addProjectAlarmRuleObjRelDTO.setObjId(updateAlarmItemDTO.getObjId());
            addProjectAlarmRuleObjRelDTO.setProjectId(projectId);
            addProjectAlarmRuleObjRelDTO.setOpen(1);
            addProjectAlarmRuleObjRelDTO.setObjName(updateAlarmItemDTO.getObjName());
            list.add(addProjectAlarmRuleObjRelDTO);
        }
        return list;
    }

    public String getConditionId(String alarmItemCode) {
        //查询对应报警类型的集团规则，将ID添加到项目规则表
        QueryGroupAlarmRuleDTO queryAlarmConditionDTO = new QueryGroupAlarmRuleDTO();
        queryAlarmConditionDTO.setAlarmItemCode(alarmItemCode);
        List<GroupAlarmRule> groupAlarmRuleList = groupAlarmRuleService.queryGroupAlarmRuleList(queryAlarmConditionDTO);
        String conditionId = groupAlarmRuleList.get(0).getAlarmConditionId();
        return conditionId;
    }

    /**
     * 校验项目报警规则是否可删除
     *
     * @param id 项目报警规则主键
     * @return 项目报警规则do类
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    public ProjectAlarmRule checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRule时发生异常，主键为空");
        }

        ProjectAlarmRule projectAlarmRule = getById(id);

        if (projectAlarmRule == null) {
            throw new IllegalArgumentException("删除ProjectAlarmRule时发生异常，找不到要删除的数据，id:" + id);
        }

        return projectAlarmRule;
    }

    /**
     * 删除项目报警规则
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteProjectAlarmRule(DeleteProjectAlarmRuleDTO deleteProjectAlarmRuleDTO) throws Exception {
        // 校验是否可删除
        ProjectAlarmRule projectAlarmRule = checkDeletable(deleteProjectAlarmRuleDTO.getId());
        projectAlarmRule.setValid(ValidEnum.FALSE.getType());
        updateById(projectAlarmRule);
        //查询是否是自定义报警,是则删除自定义规则
        int belong = projectAlarmRule.getBelong();
        if (belong == 0) {
            alarmConditionService.deleteAlarmCondition(projectAlarmRule.getAlarmConditionId());
        }
        //删除项目报警规则与监测对象的关联
        projectAlarmRuleObjRelService.deleteRelation(projectAlarmRule.getId());
        //删除设备报警条件 begin
        BatchDeleteAlarmConfigDTO batchDeleteAlarmConfigDTO = new BatchDeleteAlarmConfigDTO();
        List<DeleteAlarmConfigDTO> alarmConfigs = new ArrayList<>();
        String condition = deleteProjectAlarmRuleDTO.getCondition();
        String classCode = deleteProjectAlarmRuleDTO.getClassCode();
        String projectId = deleteProjectAlarmRuleDTO.getProjectId();
        //查询所有设备
        List<EquipItem> equips = this.getAllEquip(classCode, condition, projectId);
        List<String> objIds = equips.stream().map(EquipItem::getId).collect(Collectors.toList());
        QueryAlarmConfigDTO queryAlarmConfigDTO = new QueryAlarmConfigDTO();
        queryAlarmConfigDTO.setObjId(objIds);
        DmpResult<List<AlarmConfigItem>> query = alarmConfigService.query(queryAlarmConfigDTO);
        //所有设备条件
        List<AlarmConfigItem> data = query.getData();
        //组织删除参数
        if (data.size() > 0) {
            data.forEach(alarmConfigItem -> {
                DeleteAlarmConfigDTO deleteAlarmConfigDTO = new DeleteAlarmConfigDTO();
                deleteAlarmConfigDTO.setId(alarmConfigItem.getId());
                alarmConfigs.add(deleteAlarmConfigDTO);
            });
            batchDeleteAlarmConfigDTO.setAlarmConfigs(alarmConfigs);
            alarmConfigService.batchDelete(batchDeleteAlarmConfigDTO);
        }
        //删除设备报警条件 end
    }

    private List<EquipItem> getAllEquip(String classCode, String condition, String projectId) {
        QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
        queryEquipDTO.setProjectId(projectId);
        queryEquipDTO.setClassCode(classCode);
        queryEquipDTO.setCondition(condition);
        List<EquipItem> record = groupManagementService.getEquipList(queryEquipDTO);
        return record;
    }

    /**
     * 查询项目报警规则
     *
     * @return List<ProjectAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public List<ProjectAlarmRule> queryProjectAlarmRuleList(QueryProjectAlarmRuleDTO queryProjectAlarmRuleDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRule.PROP_CREATIONTIME);

        if (queryProjectAlarmRuleDTO != null) {

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getGroupId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_GROUP_ID, queryProjectAlarmRuleDTO.getGroupId());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getAlarmItemCode())) {
                queryWrapper.like(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, queryProjectAlarmRuleDTO.getAlarmItemCode());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getAlarmConditionId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_ALARM_CONDITION_ID, queryProjectAlarmRuleDTO.getAlarmConditionId());
            }

            if (queryProjectAlarmRuleDTO.getBelong() != null) {
                queryWrapper.eq(ProjectAlarmRule.PROP_BELONG, queryProjectAlarmRuleDTO.getBelong());
            }

            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryProjectAlarmRuleDTO.getProjectId())) {
                queryWrapper.like(ProjectAlarmRule.PROP_PROJECT_ID, queryProjectAlarmRuleDTO.getProjectId());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询项目报警规则
     *
     * @return IPage<ProjectAlarmRule>
     * @author lixing
     * @version V1.0 2021-09-08 22:46:16
     */
    @Override
    public IPage<ProjectAlarmRule> pageQueryProjectAlarmRule(
            PageQueryProjectAlarmRuleDTO
                    pageQueryProjectAlarmRuleDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<ProjectAlarmRule> pageParam = new Page<>(pageQueryProjectAlarmRuleDTO.getPage(), pageQueryProjectAlarmRuleDTO.getSize(), true);
        // 排序信息
        //        if(CollectionUtils.isEmpty(pageQueryProjectAlarmRuleDTO.getOrders())){
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, ProjectAlarmRule.PROP_CREATIONTIME);
        /*}else {
            List<Sort> orders = pageQueryProjectAlarmRuleDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }*/

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getGroupId())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_GROUP_ID, pageQueryProjectAlarmRuleDTO.getGroupId());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getAlarmItemCode())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, pageQueryProjectAlarmRuleDTO.getAlarmItemCode());
        }

        // todo 需判断使用like还是eq
        /*if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getAlarmConditionId())) {
            queryWrapper.like(ProjectAlarmRule.PROP_ALARM_CONDITION_ID, pageQueryProjectAlarmRuleDTO.getAlarmConditionId());
        }*/

        //        if (pageQueryProjectAlarmRuleDTO.getBelong() != null) {
        //            queryWrapper.eq(ProjectAlarmRule.PROP_BELONG, pageQueryProjectAlarmRuleDTO.getBelong());
        //        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryProjectAlarmRuleDTO.getProjectId())) {
            queryWrapper.eq(ProjectAlarmRule.PROP_PROJECT_ID, pageQueryProjectAlarmRuleDTO.getProjectId());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public List<AddProjectAlarmRuleObjRelDTO> addProjectAlarmRules(AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) {
        String condition = addGroupAlarmRuleDTO.getCondition();
        String classCode = addGroupAlarmRuleDTO.getClassCode();
        //查询所有项目
        List<ProjectVO> projects = this.getProject();
        List<EquipItem> records = new ArrayList<>();
        for (ProjectVO project : projects) {
            QueryEquipDTO queryEquipDTO = new QueryEquipDTO();
            queryEquipDTO.setProjectId("Pj" + project.getProjectId());
            queryEquipDTO.setClassCode(classCode);
            queryEquipDTO.setCondition(condition);
            List<EquipItem> record = groupManagementService.getEquipList(queryEquipDTO);
            records.addAll(record);
        }
        //遍历项目，组好对象进行插入
        List<ProjectAlarmRule> projectAlarmRuleList = new ArrayList<>();
        String alarmItemCode = addGroupAlarmRuleDTO.getAlarmItemCode();
        String alarmItemName = addGroupAlarmRuleDTO.getAlarmItemName();
        List<AddProjectAlarmRuleObjRelDTO> addRelationList = new ArrayList<>();
        for (ProjectVO project : projects) {
            ProjectAlarmRule projectAlarmRule = new ProjectAlarmRule();
            projectAlarmRule.setProjectId("Pj" + project.getProjectId());
            projectAlarmRule.setAlarmItemCode(alarmItemCode);
            projectAlarmRule.setAlarmItemName(alarmItemName);
            projectAlarmRule.setBelong(INTEGER_ONE);
            projectAlarmRule.setGroupId(addGroupAlarmRuleDTO.getGroupId());
            projectAlarmRule.setId(DataUtils.getUUID());
            String conditionId = getConditionId(alarmItemCode);
            projectAlarmRule.setAlarmConditionId(conditionId);
            setDefaultValue(projectAlarmRule);
            projectAlarmRuleList.add(projectAlarmRule);
            //新增关联关系
            for (EquipItem record : records) {
                AddProjectAlarmRuleObjRelDTO addProjectAlarmRuleObjRelDTO = new AddProjectAlarmRuleObjRelDTO();
                addProjectAlarmRuleObjRelDTO.setProjectAlarmRuleId(projectAlarmRule.getId());
                addProjectAlarmRuleObjRelDTO.setProjectId("Pj" + project.getProjectId());
                addProjectAlarmRuleObjRelDTO.setObjId(record.getId());
                addProjectAlarmRuleObjRelDTO.setObjName(record.getName());
                addProjectAlarmRuleObjRelDTO.setOpen(INTEGER_ONE);
                addRelationList.add(addProjectAlarmRuleObjRelDTO);
            }
        }
        projectAlarmRuleObjRelService.createBatchRelation(addRelationList);
        saveBatch(projectAlarmRuleList);
        return addRelationList;
    }

    @Override
    public List<String> queryAlarmRule(QueryRuleObjRelDTO queryRuleObjRelDTO) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_GROUP_ID, queryRuleObjRelDTO.getGroupId());
        queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, queryRuleObjRelDTO.getAlarmItemCode());
        queryWrapper.eq(ProjectAlarmRule.PROP_PROJECT_ID, queryRuleObjRelDTO.getProjectId());
        queryWrapper.eq(ProjectAlarmRule.PROP_VALID, ValidEnum.TRUE.getType());
        List<ProjectAlarmRule> list = list(queryWrapper);
        List<String> idList = list.stream().map(ProjectAlarmRule::getId).collect(Collectors.toList());
        return idList;
    }

    @Override
    public List<ProjectVO> getProject() {
        return mapper.getProject();
    }

    @Override
    public void insertProject(String substring) {
        mapper.insertProject(substring);
    }

    @Override
    public void sendBatchCreateAlarmConfigMsg(List<AddAlarmConfigDTO> alarmConfigs, String projectId) {
        if (CollectionUtils.isEmpty(alarmConfigs)) {
            return;
        }
        DmpMessage dmpMessage = new DmpMessage();
        dmpMessage.setGroupCode(PoemsContext.getContext().getGroupCode());
        dmpMessage.setProjectId(projectId);
        dmpMessage.setUserId(PoemsContext.getContext().getUserId());
        dmpMessage.setAppId("energy-alarm-service");
        dmpMessage.setType("alarmConfigBatchCreate");
        JSONObject exts = new JSONObject();
        exts.put("alarmConfigs", alarmConfigs);
        dmpMessage.setExts(exts);
        rabbitMsgSender.convertAndSend(dmpMessage);
    }

    @Override
    public void deleteBatch(GroupAlarmRule groupAlarmRule) {
        QueryWrapper<ProjectAlarmRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(ProjectAlarmRule.PROP_GROUP_ID, groupAlarmRule.getGroupId());
        queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_ITEM_CODE, groupAlarmRule.getAlarmItemCode());
        queryWrapper.eq(ProjectAlarmRule.PROP_ALARM_CONDITION_ID, groupAlarmRule.getAlarmConditionId());
        List<ProjectAlarmRule> list = list(queryWrapper);
        ProjectAlarmRule projectAlarmRule = new ProjectAlarmRule();
        projectAlarmRule.setValid(ValidEnum.FALSE.getType());
        update(projectAlarmRule,queryWrapper);
    }

    /**
     * 利用subList方法进行分页
     *
     * @param list        分页数据
     * @param pagesize    页面大小
     * @param currentPage 当前页面
     * @return
     */
    @Override
    public List<ProjectAlarmRuleListItemVO> pageBySubList(List<ProjectAlarmRuleListItemVO> list, int pagesize, int currentPage) {
        if ((currentPage - 1) * pagesize > list.size()) {
            return new ArrayList<>();
        }
        return list.subList((currentPage - 1) * pagesize, Math.min(currentPage * pagesize, list.size()));
    }
}
